<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" 
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:func="http://oxygenxml.com/samples/books"
    exclude-result-prefixes="xs xd"
    xpath-default-namespace="http://oxygenxml.com/samples/books" 
    version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> May 8, 2013</xd:p>
            <xd:p><xd:b>Author:</xd:b> Octavian Nadolu</xd:p>
            <xd:p>Generates a PDF file containing a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    
    <xsl:include href="commons/params.xsl"/>
    <xsl:include href="commons/common.xsl"/>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="books">
        <fo:root>
            <fo:layout-master-set>
                <!-- A4 size -->
                <fo:simple-page-master master-name="books" page-height="29.7cm" page-width="21cm">
                    <fo:region-body margin="1in 1in"/>
                    <fo:region-before extent="1in" display-align="after"/>
                    <fo:region-after extent="1in" display-align="after"/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            <fo:page-sequence master-reference="books">
                <fo:flow flow-name="xsl-region-body" font="8pt Times">
                    <fo:block font="20pt Times" padding="12pt" font-weight="bold">
                        The list of recommended books
                    </fo:block>
                    <xsl:call-template name="processMyBooks">
                        <xsl:with-param name="generateIndex" select="$generateIndex"/>
                        <xsl:with-param name="groupByTechnology" select="$groupByTechnology"/>
                    </xsl:call-template>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>
    </xsl:template>

    <xsl:template name="generateTable">
        <xsl:param name="books"/>
        <fo:block padding="9pt">
            <fo:table width="100%">
                <fo:table-column column-width="proportional-column-width(1)"/>
                <fo:table-column column-width="proportional-column-width(1)"/>
                <fo:table-body>
                    <xsl:apply-templates select="$books"/>
                </fo:table-body>
            </fo:table>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates the a title for the given book category.</xd:p>
        </xd:desc>
        <xd:param name="category">The name of the book category.</xd:param>
    </xd:doc>
    <xsl:template name="generateCategoryTitle">
        <xsl:param name="category"/>
        <fo:block padding="9pt">
            <fo:inline font-size="200%" font-weight="bold">
                <xsl:value-of select="$category"/>
            </fo:inline>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a link for a book the index.</xd:p>
        </xd:desc>
        <xd:param name="bookAnchor">The anchor of the reference.</xd:param>
        <xd:param name="bookName">The text displayed for the link.</xd:param>
        <xd:param name="bookAuthor">The author of the book.</xd:param>
    </xd:doc>
    <xsl:template name="createIndexItem">
        <xsl:param name="bookAnchor"/>
        <xsl:param name="bookName"/>
        <xsl:param name="bookAuthor"/>
        <fo:block font-size="100%">
            <fo:basic-link internal-destination="{$bookAnchor}" color="blue"
                text-decoration="underline">
                <xsl:value-of select="$bookName"/>
            </fo:basic-link>
            <fo:inline font-style="italic"> - by <xsl:value-of select="$bookAuthor"/>
            </fo:inline>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Creates a row with two columns for each book.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="book">
        <!-- Book -->
        <fo:table-row>
            <fo:table-cell width="110px">
                <!--Image-->
                <xsl:apply-templates select="image"/>
            </fo:table-cell>
            <fo:table-cell>
                <fo:block id="{@anchor}" padding="3pt">
                    <!-- Title with link -->
                    <xsl:variable name="title" select="title"/>
                    <xsl:variable name="titleURI" select="@targeturi"/>
                    
                    <xsl:call-template name="linkTitle">
                        <xsl:with-param name="titleUri" select="$titleURI"/>
                        <xsl:with-param name="title" select="$title"/>
                    </xsl:call-template>
                    <!-- Author -->
                    <fo:block>
                        <fo:inline font-style="italic">by 
                            <xsl:value-of select="author"/>
                        </fo:inline>
                    </fo:block>
                    <!-- Book info -->
                    <fo:block>
                        <fo:inline font-weight="bold">Paperback: </fo:inline><xsl:value-of select="paperback"/> | 
                        <fo:inline font-weight="bold">Publisher: </fo:inline><xsl:value-of select="publisher"/> | 
                        <fo:inline font-weight="bold">Isbn: </fo:inline><xsl:value-of select="isbn"/>
                    </fo:block>
                    <!-- Description -->
                    <xsl:apply-templates select="description"/>
                </fo:block>
            </fo:table-cell>
        </fo:table-row>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Creates an image element in a block.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="image">
        <fo:block space-after="10pt">
            <fo:external-graphic src="url('{@src}')" content-height="scale-to-fit"
                scaling="non-uniform" content-width="100px" width="100px" height="140px"/>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a title with link</xd:p>
        </xd:desc>
        <xd:param name="titleUri">The URI of the title</xd:param>
        <xd:param name="title">The value of the title</xd:param>
    </xd:doc>
    <xsl:template name="linkTitle">
        <xsl:param name="titleUri"/>
        <xsl:param name="title"/>
        <fo:block font-size="150%">
            <fo:basic-link external-destination="url('{$titleUri}')" color="blue"
                text-decoration="underline">
                <xsl:value-of select="$title"/>
            </fo:basic-link>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a paragraph element.</xd:desc>
    </xd:doc>
    <xsl:template match="para">
        <fo:block>
            <xsl:apply-templates/>
        </fo:block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates an italic element.</xd:desc>
    </xd:doc>
    <xsl:template match="italic">
        <fo:inline font-style="italic">
            <xsl:apply-templates/>
        </fo:inline>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a list element.</xd:desc>
    </xd:doc>
    <xsl:template match="list">
        <fo:list-block start-indent="20pt">
            <xsl:apply-templates/>
        </fo:list-block>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a list item element.</xd:desc>
    </xd:doc>
    <xsl:template match="element">
        <fo:list-item>
            <fo:list-item-label end-indent="label-end()">
                <fo:block font-weight="bold">
                    <fo:character character="&#x2022;"/>
                </fo:block>
            </fo:list-item-label>
            <fo:list-item-body start-indent="body-start()">
                <fo:block>
                    <xsl:apply-templates/>
                </fo:block>
            </fo:list-item-body>
        </fo:list-item>
    </xsl:template>
</xsl:stylesheet>
